#!/bin/bash
export PATH=./usr/bin:$PATH
# set -x

cmd=$1
# init
# stop
# test
# cleanup

# set up 2 syncthing instances lostening at
# 127.0.1.1:8384
# 127.0.2.1:8384
# with home in $PREFIX/st1-home and $PREFIX/st2-home
#

# set up current working dir as workfolder
PREFIX=$(pwd)

function st_init () {
  loginsert "st_init () start"

  mkdir -vp $PREFIX
  cd $PREFIX

  declare -a DEVID
  for ID in 1 2; do
    syncthing generate --home=st${ID}-home --no-default-folder --skip-port-probing

    # force change gui adress
    local _sed=(
        # force change gui adress
        -e 's+127.0.0.1:8384+127.0.'$ID'.1:8384+'
        # silence chatter
        -e 's+<globalAnnounceEnabled>true+<globalAnnounceEnabled>false+'
        -e 's+<localAnnounceEnabled>true+<localAnnounceEnabled>false+'
        # we don't need relaying
        -e 's+<relaysEnabled>true+<relaysEnabled>false+'
        # tun neither
        -e 's+<stunServer>default+<stunServer>127.0.0.255+'
        # close ur requester
        -e 's+<urAccepted>0+<urAccepted>-1+'
        # yes we know we don't have a secure password
        -e 's+<unackedNotificationID>authenticationUserAndPassword+<unackedNotificationID>+'
        # listen address
        -e 's+<listenAddress>default+<listenAddress>tcp://127.0.'$ID'.1:22000+'
    )

    sed -i "${_sed[@]}" st${ID}-home/config.xml

    STTRACE=all \
    syncthing serve --home=st${ID}-home \
        --logfile=- \
        --logflags=19 \
        --no-browser \
        --no-restart \
        --paused \
        --verbose \
        2>&1 >> st$ID.log &

#    echo $# > st$ID.pid

    # wait until syncthing is talking ( side effect: set default path )
    while `true`; do
        syncthing cli --home=st${ID}-home config defaults folder path set $PREFIX/st${ID}-default
        if [ $? == 0 ]; then
            break
        fi
        echo "syncthing #$ID starting up, retrying..." >&2
        sleep 2
    done
    # up and runing, so get an individual name
    DEVID[$ID]=$(syncthing --home=st${ID}-home --device-id)
    syncthing cli --home=st${ID}-home config devices ${DEVID[$ID]} name set ST$ID

  done

  loginsert "st_init () done"
}

st_test () {
    loginsert "st_test () start"
    declare -a DEVID
    for ID in 1 2; do
        # wait until syncthing is talking
        while `true`; do
            set +e
            DEVID[$ID]=$(syncthing --home=st${ID}-home --device-id)
            set -e
            if [ $? == 0 ]; then
                break
            fi
            echo "syncthing #$ID still offline, retrying..." >&2
            sleep 2
        done
    done

    # give the other syncthing a name and connection
    syncthing cli --home=st1-home config devices add --device-id ${DEVID[2]} --name ST2
    syncthing cli --home=st1-home config devices ${DEVID[2]} addresses add tcp://127.0.2.1:22000
    syncthing cli --home=st2-home config devices add --device-id ${DEVID[1]} --name ST1
    syncthing cli --home=st2-home config devices ${DEVID[1]} addresses add tcp://127.0.1.1:22000

    # add & create folder for ST1
    FOLDER=A

    syncthing cli --home=st1-home config folders add --path $PREFIX/st1-default/${FOLDER} --id ${FOLDER}-folderid --label ${FOLDER}

    # create something to share
    echo "TESTFILE FROM ST1" >${PREFIX}/st1-default/${FOLDER}/testfile-from-st1

    loginsert "ST1 folder A adding device ST2 ..."
    # share folder with ST2
    syncthing cli --home=st1-home config folders ${FOLDER}-folderid devices add --device-id ${DEVID[2]}

    # notification pops up on ST2
    loginsert "ST1 folder A has been told to share A with ST2"

    # st2 add
    # st2 save
    # st2 edit
    # st2 remove

    # st1 pause an
    # st1 pause aus  ! wichtig !
    # notification pops up on ST2
    # st2 add
    # st2 change path ( append '2' )
    # st2 save
    loginsert "st_test () done"

}


loginsert () {
  T=$(date +'%+4Y/%m/%d %T.%N')
  for ID in 1 2; do
     echo "#### ${T:0:27} ${@}" >> st${ID}.log
  done
}

st_stop () {
  for ID in 1 2; do
     syncthing cli --home=st${ID}-home operations shutdown
  done
}

st_cleanup () {
  st_stop
  sleep 2
  pwd
  for ID in 1 2; do
     rm -rf st$ID
     rm -rf st$ID-*
     rm -v st$ID.log
  done
}

set -x
case "$1" in
  init)
    st_cleanup
    st_init
  ;;
  test)
    st_test
  ;;
  cleanup)
    st_cleanup
  ;;
  stop)
    st_stop
  ;;
  *)
    echo "usage: $0 {init|config|stop}" >&2
    exit 1;
    ;;

esac


